from abc import ABC, abstractmethod
from typing import Any, cast

from google.protobuf.message import Message

from .filter import FilterWrapper, StringFilter, DurationFilter, NumberFilter, FType, _na

from ...protogen.common.tasks_fields_pb2 import (
    TaskField as rawTaskField,
    TaskSummaryField,
    TaskOptionGenericField,
    TaskOptionField,
    TASK_SUMMARY_ENUM_FIELD_TASK_ID,
    TASK_SUMMARY_ENUM_FIELD_SESSION_ID,
    TASK_SUMMARY_ENUM_FIELD_OWNER_POD_ID,
    TASK_SUMMARY_ENUM_FIELD_INITIAL_TASK_ID,
    TASK_SUMMARY_ENUM_FIELD_CREATED_BY,
    TASK_SUMMARY_ENUM_FIELD_STATUS,
    TASK_SUMMARY_ENUM_FIELD_CREATED_AT,
    TASK_SUMMARY_ENUM_FIELD_SUBMITTED_AT,
    TASK_SUMMARY_ENUM_FIELD_STARTED_AT,
    TASK_SUMMARY_ENUM_FIELD_ENDED_AT,
    TASK_SUMMARY_ENUM_FIELD_CREATION_TO_END_DURATION,
    TASK_SUMMARY_ENUM_FIELD_PROCESSING_TO_END_DURATION,
    TASK_SUMMARY_ENUM_FIELD_RECEIVED_TO_END_DURATION,
    TASK_SUMMARY_ENUM_FIELD_POD_TTL,
    TASK_SUMMARY_ENUM_FIELD_POD_HOSTNAME,
    TASK_SUMMARY_ENUM_FIELD_RECEIVED_AT,
    TASK_SUMMARY_ENUM_FIELD_ACQUIRED_AT,
    TASK_SUMMARY_ENUM_FIELD_PROCESSED_AT,
    TASK_SUMMARY_ENUM_FIELD_ERROR,
    TASK_SUMMARY_ENUM_FIELD_FETCHED_AT,
    TASK_SUMMARY_ENUM_FIELD_PAYLOAD_ID,
    TASK_OPTION_ENUM_FIELD_MAX_DURATION,
    TASK_OPTION_ENUM_FIELD_MAX_RETRIES,
    TASK_OPTION_ENUM_FIELD_PRIORITY,
    TASK_OPTION_ENUM_FIELD_PARTITION_ID,
    TASK_OPTION_ENUM_FIELD_APPLICATION_NAME,
    TASK_OPTION_ENUM_FIELD_APPLICATION_NAMESPACE,
    TASK_OPTION_ENUM_FIELD_APPLICATION_VERSION,
    TASK_OPTION_ENUM_FIELD_APPLICATION_SERVICE,
    TASK_OPTION_ENUM_FIELD_ENGINE_TYPE,
)

from ...protogen.common.sessions_fields_pb2 import (
    SessionField,
    SessionRawField,
    SESSION_RAW_ENUM_FIELD_SESSION_ID,
    SESSION_RAW_ENUM_FIELD_DURATION,
    SESSION_RAW_ENUM_FIELD_CREATED_AT,
    SESSION_RAW_ENUM_FIELD_DELETED_AT,
    SESSION_RAW_ENUM_FIELD_PURGED_AT,
    SESSION_RAW_ENUM_FIELD_STATUS,
    SESSION_RAW_ENUM_FIELD_PARTITION_IDS,
    SESSION_RAW_ENUM_FIELD_CLIENT_SUBMISSION,
    SESSION_RAW_ENUM_FIELD_WORKER_SUBMISSION,
    SESSION_RAW_ENUM_FIELD_CLOSED_AT,
    SESSION_RAW_ENUM_FIELD_CANCELLED_AT,
    TaskOptionField as SessionOptionField,
    TaskOptionGenericField as SessionOptionGenericField,
    TASK_OPTION_ENUM_FIELD_MAX_RETRIES as SESSION_OPTION_MAX_RETRIES,
    TASK_OPTION_ENUM_FIELD_APPLICATION_NAMESPACE as SESSION_OPTION_APPLICATION_NAMESPACE,
    TASK_OPTION_ENUM_FIELD_APPLICATION_VERSION as SESSION_OPTION_APPLICATION_VERSION,
    TASK_OPTION_ENUM_FIELD_PRIORITY as SESSION_OPTION_PRIORITY,
    TASK_OPTION_ENUM_FIELD_APPLICATION_NAME as SESSION_OPTION_APPLICATION_NAME,
    TASK_OPTION_ENUM_FIELD_PARTITION_ID as SESSION_OPTION_PARTITION_ID,
    TASK_OPTION_ENUM_FIELD_ENGINE_TYPE as SESSION_OPTION_ENGINE_TYPE,
    TASK_OPTION_ENUM_FIELD_MAX_DURATION as SESSION_OPTION_MAX_DURATION,
    TASK_OPTION_ENUM_FIELD_APPLICATION_SERVICE as SESSION_OPTION_APPLICATION_SERVICE,
)

from ...protogen.common.results_fields_pb2 import (
    ResultField,
    ResultRawField,
    RESULT_RAW_ENUM_FIELD_RESULT_ID,
    RESULT_RAW_ENUM_FIELD_SIZE,
    RESULT_RAW_ENUM_FIELD_NAME,
    RESULT_RAW_ENUM_FIELD_STATUS,
    RESULT_RAW_ENUM_FIELD_COMPLETED_AT,
    RESULT_RAW_ENUM_FIELD_CREATED_AT,
    RESULT_RAW_ENUM_FIELD_SESSION_ID,
    RESULT_RAW_ENUM_FIELD_CREATED_BY,
    RESULT_RAW_ENUM_FIELD_OWNER_TASK_ID,
)

from ...protogen.common.partitions_fields_pb2 import (
    PartitionField,
    PartitionRawField,
    PARTITION_RAW_ENUM_FIELD_ID,
    PARTITION_RAW_ENUM_FIELD_PRIORITY,
    PARTITION_RAW_ENUM_FIELD_PREEMPTION_PERCENTAGE,
    PARTITION_RAW_ENUM_FIELD_POD_RESERVED,
    PARTITION_RAW_ENUM_FIELD_PARENT_PARTITION_IDS,
    PARTITION_RAW_ENUM_FIELD_POD_MAX,
)

from ...protogen.common.applications_fields_pb2 import (
    ApplicationField,
    ApplicationRawField,
    APPLICATION_RAW_ENUM_FIELD_NAMESPACE,
    APPLICATION_RAW_ENUM_FIELD_SERVICE,
    APPLICATION_RAW_ENUM_FIELD_VERSION,
    APPLICATION_RAW_ENUM_FIELD_NAME,
)

from ...protogen.common.tasks_filters_pb2 import (
    FilterField as rawTaskFilterField,
    FilterStatus as rawTaskFilterStatus,
    Filters as rawTaskFilters,
    FiltersAnd as rawTaskFilterAnd,
)

from ...protogen.common.sessions_filters_pb2 import (
    FilterField as rawSessionFilterField,
    FilterStatus as rawSessionFilterStatus,
    Filters as rawSessionFilters,
    FiltersAnd as rawSessionFilterAnd,
)

from ...protogen.common.results_filters_pb2 import (
    FilterField as rawResultFilterField,
    FilterStatus as rawResultFilterStatus,
    Filters as rawResultFilters,
    FiltersAnd as rawResultFilterAnd,
)

from ...protogen.common.partitions_filters_pb2 import (
    FilterField as rawPartitionFilterField,
    Filters as rawPartitionFilters,
    FiltersAnd as rawPartitionFilterAnd,
)

from ...protogen.common.applications_filters_pb2 import (
    FilterField as rawApplicationFilterField,
    Filters as rawApplicationFilters,
    FiltersAnd as rawApplicationFilterAnd,
)

"""
This file defines the fields available for each object type. Used internally.
"""


class GenericTaskOptionsFilter(FilterWrapper, ABC):
    """
    Filter for task options
    """

    @abstractmethod
    def __getitem__(self, item: str) -> StringFilter: ...

    @abstractmethod
    def _build_field(self, field: Any) -> Message: ...

    @property
    def max_duration(self) -> DurationFilter:
        return cast(DurationFilter, self("max_duration"))

    @property
    def max_retries(self) -> NumberFilter:
        return cast(NumberFilter, self("max_retries"))

    @property
    def priority(self) -> NumberFilter:
        return cast(NumberFilter, self("priority"))

    @property
    def partition_id(self) -> StringFilter:
        return cast(StringFilter, self("partition_id"))

    @property
    def application_name(self) -> StringFilter:
        return cast(StringFilter, self("application_name"))

    @property
    def application_namespace(self) -> StringFilter:
        return cast(StringFilter, self("application_namespace"))

    @property
    def application_version(self) -> StringFilter:
        return cast(StringFilter, self("application_version"))

    @property
    def application_service(self) -> StringFilter:
        return cast(StringFilter, self("application_service"))

    @property
    def engine_type(self) -> StringFilter:
        return cast(StringFilter, self("engine_type"))


class OutputFilter(FilterWrapper):
    def __init__(self):
        super().__init__(rawTaskFilters, rawTaskFilterAnd, rawTaskFilterField, rawTaskFilterStatus)

    @property
    def error(self) -> StringFilter:
        return self._string(self._build_field(TASK_SUMMARY_ENUM_FIELD_ERROR))

    def _build_field(self, field: Any) -> Message:
        if field != TASK_SUMMARY_ENUM_FIELD_ERROR:
            _na("")
        return rawTaskField(task_summary_field=TaskSummaryField(field=field))


class TaskFilter(FilterWrapper):
    _fields = {
        "id": (FType.STR, TASK_SUMMARY_ENUM_FIELD_TASK_ID),
        "session_id": (FType.STR, TASK_SUMMARY_ENUM_FIELD_SESSION_ID),
        "owner_pod_id": (FType.STR, TASK_SUMMARY_ENUM_FIELD_OWNER_POD_ID),
        "initial_task_id": (FType.STR, TASK_SUMMARY_ENUM_FIELD_INITIAL_TASK_ID),
        "created_by": (FType.STR, TASK_SUMMARY_ENUM_FIELD_CREATED_BY),
        "parent_task_ids": (FType.NA, "parent_task_ids"),
        "data_dependencies": (FType.NA, "data_dependencies"),
        "expected_output_ids": (FType.NA, "expected_output_ids"),
        "retry_of_ids": (FType.NA, "retry_of_ids"),
        "status": (FType.STATUS, TASK_SUMMARY_ENUM_FIELD_STATUS),
        "status_message": (FType.NA, "status_message"),
        "options": (FType.NA, "options"),
        "created_at": (FType.DATE, TASK_SUMMARY_ENUM_FIELD_CREATED_AT),
        "submitted_at": (FType.DATE, TASK_SUMMARY_ENUM_FIELD_SUBMITTED_AT),
        "received_at": (FType.DATE, TASK_SUMMARY_ENUM_FIELD_RECEIVED_AT),
        "acquired_at": (FType.DATE, TASK_SUMMARY_ENUM_FIELD_ACQUIRED_AT),
        "fetched_at": (FType.DATE, TASK_SUMMARY_ENUM_FIELD_FETCHED_AT),
        "started_at": (FType.DATE, TASK_SUMMARY_ENUM_FIELD_STARTED_AT),
        "processed_at": (FType.DATE, TASK_SUMMARY_ENUM_FIELD_PROCESSED_AT),
        "ended_at": (FType.DATE, TASK_SUMMARY_ENUM_FIELD_ENDED_AT),
        "pod_ttl": (FType.DATE, TASK_SUMMARY_ENUM_FIELD_POD_TTL),
        "creation_to_end_duration": (
            FType.DURATION,
            TASK_SUMMARY_ENUM_FIELD_CREATION_TO_END_DURATION,
        ),
        "processing_to_end_duration": (
            FType.DURATION,
            TASK_SUMMARY_ENUM_FIELD_PROCESSING_TO_END_DURATION,
        ),
        "received_to_end_duration": (
            FType.DURATION,
            TASK_SUMMARY_ENUM_FIELD_RECEIVED_TO_END_DURATION,
        ),
        "output": (FType.NA, "output"),
        "pod_hostname": (FType.STR, TASK_SUMMARY_ENUM_FIELD_POD_HOSTNAME),
        "payload_id": (FType.STR, TASK_SUMMARY_ENUM_FIELD_PAYLOAD_ID),
    }

    def __init__(self):
        super().__init__(rawTaskFilters, rawTaskFilterAnd, rawTaskFilterField, rawTaskFilterStatus)

    def _build_field(self, field: Any) -> Message:
        return rawTaskField(task_summary_field=TaskSummaryField(field=field))


class TaskOptionFilter(GenericTaskOptionsFilter):
    _fields = {
        "max_duration": (FType.DURATION, TASK_OPTION_ENUM_FIELD_MAX_DURATION),
        "max_retries": (FType.NUM, TASK_OPTION_ENUM_FIELD_MAX_RETRIES),
        "priority": (FType.NUM, TASK_OPTION_ENUM_FIELD_PRIORITY),
        "partition_id": (FType.STR, TASK_OPTION_ENUM_FIELD_PARTITION_ID),
        "application_name": (FType.STR, TASK_OPTION_ENUM_FIELD_APPLICATION_NAME),
        "application_namespace": (FType.STR, TASK_OPTION_ENUM_FIELD_APPLICATION_NAMESPACE),
        "application_version": (FType.STR, TASK_OPTION_ENUM_FIELD_APPLICATION_VERSION),
        "application_service": (FType.STR, TASK_OPTION_ENUM_FIELD_APPLICATION_SERVICE),
        "engine_type": (FType.STR, TASK_OPTION_ENUM_FIELD_ENGINE_TYPE),
    }

    def __init__(self):
        super().__init__(rawTaskFilters, rawTaskFilterAnd, rawTaskFilterField, rawTaskFilterStatus)

    def __getitem__(self, item: str) -> StringFilter:
        return self._string(
            rawTaskField(task_option_generic_field=TaskOptionGenericField(field=item))
        )

    def _build_field(self, field: Any) -> Message:
        return rawTaskField(task_option_field=TaskOptionField(field=field))


class SessionFilter(FilterWrapper):
    _fields = {
        "session_id": (FType.STR, SESSION_RAW_ENUM_FIELD_SESSION_ID),
        "status": (FType.STATUS, SESSION_RAW_ENUM_FIELD_STATUS),
        "client_submission": (FType.BOOL, SESSION_RAW_ENUM_FIELD_CLIENT_SUBMISSION),
        "worker_submission": (FType.BOOL, SESSION_RAW_ENUM_FIELD_WORKER_SUBMISSION),
        "partition_ids": (FType.ARRAY, SESSION_RAW_ENUM_FIELD_PARTITION_IDS),
        "options": (FType.NA, "options"),
        "created_at": (FType.DATE, SESSION_RAW_ENUM_FIELD_CREATED_AT),
        "cancelled_at": (FType.DATE, SESSION_RAW_ENUM_FIELD_CANCELLED_AT),
        "closed_at": (FType.DATE, SESSION_RAW_ENUM_FIELD_CLOSED_AT),
        "purged_at": (FType.DATE, SESSION_RAW_ENUM_FIELD_PURGED_AT),
        "deleted_at": (FType.DATE, SESSION_RAW_ENUM_FIELD_DELETED_AT),
        "duration": (FType.DURATION, SESSION_RAW_ENUM_FIELD_DURATION),
    }

    def __init__(self):
        super().__init__(
            rawSessionFilters, rawSessionFilterAnd, rawSessionFilterField, rawSessionFilterStatus
        )

    def _build_field(self, field: Any) -> Message:
        return SessionField(session_raw_field=SessionRawField(field=field))


class SessionTaskOptionFilter(GenericTaskOptionsFilter):
    _fields = {
        "max_duration": (FType.DURATION, SESSION_OPTION_MAX_DURATION),
        "max_retries": (FType.NUM, SESSION_OPTION_MAX_RETRIES),
        "priority": (FType.NUM, SESSION_OPTION_PRIORITY),
        "partition_id": (FType.STR, SESSION_OPTION_PARTITION_ID),
        "application_name": (FType.STR, SESSION_OPTION_APPLICATION_NAME),
        "application_namespace": (FType.STR, SESSION_OPTION_APPLICATION_NAMESPACE),
        "application_version": (FType.STR, SESSION_OPTION_APPLICATION_VERSION),
        "application_service": (FType.STR, SESSION_OPTION_APPLICATION_SERVICE),
        "engine_type": (FType.STR, SESSION_OPTION_ENGINE_TYPE),
    }

    def __init__(self):
        super().__init__(
            rawSessionFilters, rawSessionFilterAnd, rawSessionFilterField, rawSessionFilterStatus
        )

    def __getitem__(self, item: str) -> StringFilter:
        return self._string(
            SessionField(task_option_generic_field=SessionOptionGenericField(field=item))
        )

    def _build_field(self, field: Any) -> Message:
        return SessionField(task_option_field=SessionOptionField(field=field))


class ResultFilter(FilterWrapper):
    _fields = {
        "session_id": (FType.STR, RESULT_RAW_ENUM_FIELD_SESSION_ID),
        "status": (FType.STATUS, RESULT_RAW_ENUM_FIELD_STATUS),
        "name": (FType.STR, RESULT_RAW_ENUM_FIELD_NAME),
        "created_at": (FType.DATE, RESULT_RAW_ENUM_FIELD_CREATED_AT),
        "completed_at": (FType.DATE, RESULT_RAW_ENUM_FIELD_COMPLETED_AT),
        "result_id": (FType.STR, RESULT_RAW_ENUM_FIELD_RESULT_ID),
        "size": (FType.NUM, RESULT_RAW_ENUM_FIELD_SIZE),
        "created_by": (FType.STR, RESULT_RAW_ENUM_FIELD_CREATED_BY),
        "owner_task_id": (FType.STR, RESULT_RAW_ENUM_FIELD_OWNER_TASK_ID),
        "opaque_id": (FType.NA, "opaque_id"),
    }

    def __init__(self):
        super().__init__(
            rawResultFilters, rawResultFilterAnd, rawResultFilterField, rawResultFilterStatus
        )

    def _build_field(self, field: Any) -> Message:
        return ResultField(result_raw_field=ResultRawField(field=field))


class PartitionFilter(FilterWrapper):
    _fields = {
        "id": (FType.STR, PARTITION_RAW_ENUM_FIELD_ID),
        "priority": (FType.NUM, PARTITION_RAW_ENUM_FIELD_PRIORITY),
        "preemption_percentage": (FType.NUM, PARTITION_RAW_ENUM_FIELD_PREEMPTION_PERCENTAGE),
        "pod_reserved": (FType.NUM, PARTITION_RAW_ENUM_FIELD_POD_RESERVED),
        "pod_max": (FType.NUM, PARTITION_RAW_ENUM_FIELD_POD_MAX),
        "parent_partition_ids": (FType.ARRAY, PARTITION_RAW_ENUM_FIELD_PARENT_PARTITION_IDS),
        "pod_configuration": (FType.NA, "pod_configuration"),
    }

    def __init__(self):
        super().__init__(rawPartitionFilters, rawPartitionFilterAnd, rawPartitionFilterField)

    def _build_field(self, field: Any) -> Message:
        return PartitionField(partition_raw_field=PartitionRawField(field=field))


class ApplicationFilter(FilterWrapper):
    _fields = {
        "name": (FType.STR, APPLICATION_RAW_ENUM_FIELD_NAME),
        "namespace": (FType.STR, APPLICATION_RAW_ENUM_FIELD_NAMESPACE),
        "service": (FType.STR, APPLICATION_RAW_ENUM_FIELD_SERVICE),
        "version": (FType.STR, APPLICATION_RAW_ENUM_FIELD_VERSION),
    }

    def __init__(self):
        super().__init__(rawApplicationFilters, rawApplicationFilterAnd, rawApplicationFilterField)

    def _build_field(self, field: Any) -> Message:
        return ApplicationField(application_field=ApplicationRawField(field=field))



from . import partitions_fields_pb2 as _partitions_fields_pb2
from . import partitions_filters_pb2 as _partitions_filters_pb2
from . import sort_direction_pb2 as _sort_direction_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class PartitionRaw(_message.Message):
    __slots__ = ("id", "parent_partition_ids", "pod_reserved", "pod_max", "pod_configuration", "preemption_percentage", "priority")
    class PodConfigurationEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    ID_FIELD_NUMBER: _ClassVar[int]
    PARENT_PARTITION_IDS_FIELD_NUMBER: _ClassVar[int]
    POD_RESERVED_FIELD_NUMBER: _ClassVar[int]
    POD_MAX_FIELD_NUMBER: _ClassVar[int]
    POD_CONFIGURATION_FIELD_NUMBER: _ClassVar[int]
    PREEMPTION_PERCENTAGE_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_FIELD_NUMBER: _ClassVar[int]
    id: str
    parent_partition_ids: _containers.RepeatedScalarFieldContainer[str]
    pod_reserved: int
    pod_max: int
    pod_configuration: _containers.ScalarMap[str, str]
    preemption_percentage: int
    priority: int
    def __init__(self, id: _Optional[str] = ..., parent_partition_ids: _Optional[_Iterable[str]] = ..., pod_reserved: _Optional[int] = ..., pod_max: _Optional[int] = ..., pod_configuration: _Optional[_Mapping[str, str]] = ..., preemption_percentage: _Optional[int] = ..., priority: _Optional[int] = ...) -> None: ...

class ListPartitionsRequest(_message.Message):
    __slots__ = ("page", "page_size", "filters", "sort")
    class Sort(_message.Message):
        __slots__ = ("field", "direction")
        FIELD_FIELD_NUMBER: _ClassVar[int]
        DIRECTION_FIELD_NUMBER: _ClassVar[int]
        field: _partitions_fields_pb2.PartitionField
        direction: _sort_direction_pb2.SortDirection
        def __init__(self, field: _Optional[_Union[_partitions_fields_pb2.PartitionField, _Mapping]] = ..., direction: _Optional[_Union[_sort_direction_pb2.SortDirection, str]] = ...) -> None: ...
    PAGE_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    FILTERS_FIELD_NUMBER: _ClassVar[int]
    SORT_FIELD_NUMBER: _ClassVar[int]
    page: int
    page_size: int
    filters: _partitions_filters_pb2.Filters
    sort: ListPartitionsRequest.Sort
    def __init__(self, page: _Optional[int] = ..., page_size: _Optional[int] = ..., filters: _Optional[_Union[_partitions_filters_pb2.Filters, _Mapping]] = ..., sort: _Optional[_Union[ListPartitionsRequest.Sort, _Mapping]] = ...) -> None: ...

class ListPartitionsResponse(_message.Message):
    __slots__ = ("partitions", "page", "page_size", "total")
    PARTITIONS_FIELD_NUMBER: _ClassVar[int]
    PAGE_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    TOTAL_FIELD_NUMBER: _ClassVar[int]
    partitions: _containers.RepeatedCompositeFieldContainer[PartitionRaw]
    page: int
    page_size: int
    total: int
    def __init__(self, partitions: _Optional[_Iterable[_Union[PartitionRaw, _Mapping]]] = ..., page: _Optional[int] = ..., page_size: _Optional[int] = ..., total: _Optional[int] = ...) -> None: ...

class GetPartitionRequest(_message.Message):
    __slots__ = ("id",)
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class GetPartitionResponse(_message.Message):
    __slots__ = ("partition",)
    PARTITION_FIELD_NUMBER: _ClassVar[int]
    partition: PartitionRaw
    def __init__(self, partition: _Optional[_Union[PartitionRaw, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: partitions_filters.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import filters_common_pb2 as filters__common__pb2
from . import partitions_fields_pb2 as partitions__fields__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18partitions_filters.proto\x12\x1e\x61rmonik.api.grpc.v1.partitions\x1a\x14\x66ilters_common.proto\x1a\x17partitions_fields.proto\"\xcf\x02\n\x0b\x46ilterField\x12=\n\x05\x66ield\x18\x01 \x01(\x0b\x32..armonik.api.grpc.v1.partitions.PartitionField\x12:\n\rfilter_string\x18\x02 \x01(\x0b\x32!.armonik.api.grpc.v1.FilterStringH\x00\x12:\n\rfilter_number\x18\x03 \x01(\x0b\x32!.armonik.api.grpc.v1.FilterNumberH\x00\x12<\n\x0e\x66ilter_boolean\x18\x04 \x01(\x0b\x32\".armonik.api.grpc.v1.FilterBooleanH\x00\x12\x38\n\x0c\x66ilter_array\x18\x05 \x01(\x0b\x32 .armonik.api.grpc.v1.FilterArrayH\x00\x42\x11\n\x0fvalue_condition\"F\n\nFiltersAnd\x12\x38\n\x03\x61nd\x18\x01 \x03(\x0b\x32+.armonik.api.grpc.v1.partitions.FilterField\"A\n\x07\x46ilters\x12\x36\n\x02or\x18\x01 \x03(\x0b\x32*.armonik.api.grpc.v1.partitions.FiltersAndBI\n&fr.aneo.armonik.api.grpc.v1.partitions\xaa\x02\x1e\x41rmoniK.Api.gRPC.V1.Partitionsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'partitions_filters_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n&fr.aneo.armonik.api.grpc.v1.partitions\252\002\036ArmoniK.Api.gRPC.V1.Partitions'
  _globals['_FILTERFIELD']._serialized_start=108
  _globals['_FILTERFIELD']._serialized_end=443
  _globals['_FILTERSAND']._serialized_start=445
  _globals['_FILTERSAND']._serialized_end=515
  _globals['_FILTERS']._serialized_start=517
  _globals['_FILTERS']._serialized_end=582
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

from typing import Dict, List, Optional, Tuple

from deprecation import deprecated
from grpc import Channel

from .. import __version__
from ..common import Direction, Task, TaskDefinition, TaskOptions, TaskStatus
from ..common.filter import Filter, StringFilter, TaskFilter
from ..common.helpers import batched
from ..protogen.client.tasks_service_pb2_grpc import TasksStub
from ..protogen.common.sort_direction_pb2 import SortDirection
from ..protogen.common.tasks_common_pb2 import (
    CancelTasksRequest,
    CountTasksByStatusRequest,
    CountTasksByStatusResponse,
    GetResultIdsRequest,
    GetResultIdsResponse,
    GetTaskRequest,
    GetTaskResponse,
    ListTasksDetailedResponse,
    ListTasksRequest,
    ListTasksResponse,
    SubmitTasksRequest,
)


@deprecated("3.19.0", None, __version__, "Use Task.<name of the field> instead")
class TaskFieldFilter:
    """
    Enumeration of the available filters
    """

    TASK_ID = Task.id
    SESSION_ID = Task.session_id
    OWNER_POD_ID = Task.owner_pod_id
    INITIAL_TASK_ID = Task.initial_task_id
    STATUS = Task.status
    CREATED_AT = Task.created_at
    SUBMITTED_AT = Task.submitted_at
    STARTED_AT = Task.started_at
    ENDED_AT = Task.ended_at
    CREATION_TO_END_DURATION = Task.creation_to_end_duration
    PROCESSING_TO_END_DURATION = Task.processing_to_end_duration
    POD_TTL = Task.pod_ttl
    POD_HOSTNAME = Task.pod_hostname
    RECEIVED_AT = Task.received_at
    ACQUIRED_AT = Task.acquired_at
    ERROR = Task.output.error

    MAX_DURATION = Task.options.max_duration
    MAX_RETRIES = Task.options.max_retries
    PRIORITY = Task.options.priority
    PARTITION_ID = Task.options.partition_id
    APPLICATION_NAME = Task.options.application_name
    APPLICATION_VERSION = Task.options.application_version
    APPLICATION_NAMESPACE = Task.options.application_namespace
    APPLICATION_SERVICE = Task.options.application_service
    ENGINE_TYPE = Task.options.engine_type

    @staticmethod
    def task_options_key(option_key: str) -> StringFilter:
        """
        Filter for the TaskOptions.Options dictionary
        Args:
            option_key: key in the dictionary

        Returns:
            Corresponding filter
        """
        return Task.options[option_key]


class ArmoniKTasks:
    def __init__(self, grpc_channel: Channel):
        """Tasks service client

        Args:
            grpc_channel: gRPC channel to use
        """
        self._client = TasksStub(grpc_channel)

    def get_task(self, task_id: str) -> Task:
        """Get task information from task id

        Args:
            task_id: Id of the task

        Returns:
            Task object with the information
        """
        task_response: GetTaskResponse = self._client.GetTask(GetTaskRequest(task_id=task_id))
        return Task.from_message(task_response.task)

    def list_tasks(
        self,
        task_filter: Optional[Filter] = None,
        with_errors: bool = False,
        page: int = 0,
        page_size: int = 1000,
        sort_field: Filter = Task.id,
        sort_direction: SortDirection = Direction.ASC,
        detailed: bool = True,
    ) -> Tuple[int, List[Task]]:
        """List tasks

        If the total returned exceeds the requested page size, you may want to use this function again and ask for subsequent pages.

        Args:
            task_filter: Filter for the tasks to be listed
            with_errors: Retrieve the error if the task had errors, defaults to false
            page: page number to request, this can be useful when paginating the result, defaults to 0
            page_size: size of a page, defaults to 1000
            sort_field: field on which to sort the resulting list, defaults to the task_id
            sort_direction: direction of the sort, defaults to ascending
            detailed: Whether to retrieve the detailed description of the task.

        Returns:
            A tuple containing :
            - The total number of tasks for the given filter
            - The obtained list of tasks
        """
        request = ListTasksRequest(
            page=page,
            page_size=page_size,
            filters=(
                TaskFilter().to_message() if task_filter is None else task_filter.to_message()
            ),
            sort=ListTasksRequest.Sort(field=sort_field.field, direction=sort_direction),
            with_errors=with_errors,
        )
        if detailed:
            response: ListTasksDetailedResponse = self._client.ListTasksDetailed(request)
            return response.total, [Task.from_message(t) for t in response.tasks]
        response: ListTasksResponse = self._client.ListTasks(request)
        return response.total, [Task.from_summary(t) for t in response.tasks]

    def cancel_tasks(self, task_ids: List[str], chunk_size: Optional[int] = 500):
        """Cancel tasks.

        Args:
            task_ids: IDs of the tasks.
            chunk_size: Batch size for cancelling.

        Return:
            The list of cancelled tasks.
        """
        for task_id_batch in batched(task_ids, chunk_size):
            request = CancelTasksRequest(task_ids=task_id_batch)
            self._client.CancelTasks(request)

    def get_result_ids(
        self, task_ids: List[str], chunk_size: Optional[int] = 500
    ) -> Dict[str, List[str]]:
        """Get result IDs of a list of tasks.

        Args:
            task_ids: The IDs of the tasks.
            chunk_size: Batch size for retrieval.

        Return:
            A dictionary mapping the ID of a task to the IDs of its results..
        """
        tasks_result_ids = {}

        for task_ids_batch in batched(task_ids, chunk_size):
            request = GetResultIdsRequest(task_id=task_ids_batch)
            result_ids_response: GetResultIdsResponse = self._client.GetResultIds(request)
            for t in result_ids_response.task_results:
                tasks_result_ids[t.task_id] = list(t.result_ids)
        return tasks_result_ids

    def count_tasks_by_status(self, task_filter: Optional[Filter] = None) -> Dict[TaskStatus, int]:
        """Get number of tasks by status.

        Args:
            task_filter: Filter for the tasks to be listed

        Return:
            A dictionnary mapping each status to the number of filtered tasks.
        """
        request = CountTasksByStatusRequest(
            filters=(TaskFilter().to_message() if task_filter is None else task_filter.to_message())
        )
        count_tasks_by_status_response: CountTasksByStatusResponse = (
            self._client.CountTasksByStatus(request)
        )
        return {
            TaskStatus(status_count.status): status_count.count
            for status_count in count_tasks_by_status_response.status
        }

    def submit_tasks(
        self,
        session_id: str,
        tasks: List[TaskDefinition],
        default_task_options: Optional[TaskOptions] = None,
        chunk_size: Optional[int] = 100,
    ) -> List[Task]:
        """Submit tasks to ArmoniK.

        Args:
            session_id: Session Id
            tasks: List of task definitions
            default_task_options: Default Task Options used if a task has its options not set
            chunk_size: Batch size for submission

        Returns:
            List of successfully sent tasks
        """
        submitted = []
        for tasks_batch in batched(tasks, chunk_size):
            task_creations = []

            for t in tasks_batch:
                task_creation = SubmitTasksRequest.TaskCreation(
                    expected_output_keys=t.expected_output_ids,
                    payload_id=t.payload_id,
                    data_dependencies=t.data_dependencies,
                    task_options=t.options.to_message() if t.options else None,
                )
                task_creations.append(task_creation)

            request = SubmitTasksRequest(
                session_id=session_id,
                task_creations=task_creations,
                task_options=(default_task_options.to_message() if default_task_options else None),
            )
            submitted.extend(
                Task(
                    id=t.task_id,
                    session_id=session_id,
                    expected_output_ids=list(t.expected_output_ids),
                    data_dependencies=list(t.data_dependencies),
                    payload_id=t.payload_id,
                )
                for t in self._client.SubmitTasks(request).task_infos
            )
        return submitted



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: worker_common.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import objects_pb2 as objects__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13worker_common.proto\x12\x1a\x61rmonik.api.grpc.v1.worker\x1a\robjects.proto\"\xa7\x02\n\x0eProcessRequest\x12\x1b\n\x13\x63ommunication_token\x18\x01 \x01(\t\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\x0f\n\x07task_id\x18\x03 \x01(\t\x12\x36\n\x0ctask_options\x18\x04 \x01(\x0b\x32 .armonik.api.grpc.v1.TaskOptions\x12\x1c\n\x14\x65xpected_output_keys\x18\x05 \x03(\t\x12\x12\n\npayload_id\x18\x06 \x01(\t\x12\x19\n\x11\x64\x61ta_dependencies\x18\x07 \x03(\t\x12\x13\n\x0b\x64\x61ta_folder\x18\x08 \x01(\t\x12\x39\n\rconfiguration\x18\t \x01(\x0b\x32\".armonik.api.grpc.v1.Configuration\";\n\x0cProcessReply\x12+\n\x06output\x18\x01 \x01(\x0b\x32\x1b.armonik.api.grpc.v1.Output\"\x9a\x01\n\x10HealthCheckReply\x12J\n\x06status\x18\x01 \x01(\x0e\x32:.armonik.api.grpc.v1.worker.HealthCheckReply.ServingStatus\":\n\rServingStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07SERVING\x10\x01\x12\x0f\n\x0bNOT_SERVING\x10\x02\x42\x41\n\"fr.aneo.armonik.api.grpc.v1.worker\xaa\x02\x1a\x41rmoniK.Api.gRPC.V1.Workerb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'worker_common_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\"fr.aneo.armonik.api.grpc.v1.worker\252\002\032ArmoniK.Api.gRPC.V1.Worker'
  _globals['_PROCESSREQUEST']._serialized_start=67
  _globals['_PROCESSREQUEST']._serialized_end=362
  _globals['_PROCESSREPLY']._serialized_start=364
  _globals['_PROCESSREPLY']._serialized_end=423
  _globals['_HEALTHCHECKREPLY']._serialized_start=426
  _globals['_HEALTHCHECKREPLY']._serialized_end=580
  _globals['_HEALTHCHECKREPLY_SERVINGSTATUS']._serialized_start=522
  _globals['_HEALTHCHECKREPLY_SERVINGSTATUS']._serialized_end=580
# @@protoc_insertion_point(module_scope)



from ..common import partitions_common_pb2 as _partitions_common_pb2
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor



from __future__ import annotations

import logging
import os
import traceback
from concurrent import futures
from contextlib import nullcontext
from logging import Logger
from typing import Callable, Union, Optional, Tuple, Iterable

import grpc
from grpc import Channel

from ..common.channel import create_channel
from .seqlogger import ClefLogger
from ..common import Output, HealthCheckStatus
from ..protogen.common.objects_pb2 import Empty
from ..protogen.common.worker_common_pb2 import (
    ProcessReply,
    ProcessRequest,
    HealthCheckReply,
)
from ..protogen.worker.agent_service_pb2_grpc import AgentStub
from ..protogen.worker.worker_service_pb2_grpc import (
    WorkerServicer,
    add_WorkerServicer_to_server,
)
from .taskhandler import TaskHandler


class ArmoniKWorker(WorkerServicer):
    def __init__(
        self,
        agent_channel: Channel,
        processing_function: Callable[[TaskHandler], Output],
        health_check: Callable[
            [], HealthCheckReply.ServingStatus
        ] = lambda: HealthCheckStatus.SERVING,
        logger=ClefLogger.getLogger("ArmoniKWorker"),
    ):
        """Creates a worker for ArmoniK

        Args:
            agent_channel: gRPC channel for the agent
            processing_function: Function that will be called when a
                task needs processing
            health_check: Function that will be called to check the
                health of the worker. Defaults to a simple "Serving"
                reply
            logger: Logger used for the worker, defaults to a logger
                ArmoniKWorker compatible with Seq
        """
        self.health_check = health_check
        self.processing_function = processing_function
        self._client = AgentStub(agent_channel)
        self._logger = logger

    def start(self, endpoint: str):
        """Starts the worker

        Args:
            endpoint: endpoint from which to listen to requests
        """
        server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
        add_WorkerServicer_to_server(self, server)
        server.add_insecure_port(endpoint)
        server.start()
        server.wait_for_termination()

    def Process(self, request: ProcessRequest, context) -> Union[ProcessReply, None]:
        try:
            self._logger.debug("Received task")
            task_handler = TaskHandler(request, self._client)
            return ProcessReply(output=self.processing_function(task_handler).to_message())
        except Exception as e:
            self._logger.exception(
                f"Failed task {''.join(traceback.format_exception(type(e), e, e.__traceback__))}",
                exc_info=e,
            )

    def HealthCheck(self, request: Empty, context) -> HealthCheckReply:
        return HealthCheckReply(status=self.health_check())


class ArmoniKWorkerWrapper:
    def __init__(
        self,
        *,
        processor: Callable[[TaskHandler], Output],
        logger: Optional[Logger] = None,
        worker_endpoint: Optional[str] = None,
        agent_endpoint: Optional[str] = None,
        channel_options: Optional[Iterable[Tuple[str, str]]] = None,
    ):
        if logger is None:
            ClefLogger.setup_logging(logging.INFO)
            logger = ClefLogger.getLogger("ArmoniKWorker")
        if worker_endpoint is None:
            worker_scheme = (
                "unix://"
                if os.getenv("ComputePlane__WorkerChannel__SocketType", "unixdomainsocket")
                == "unixdomainsocket"
                else "http://"
            )
            worker_endpoint = worker_scheme + os.getenv(
                "ComputePlane__WorkerChannel__Address", "/cache/armonik_worker.sock"
            )
        if agent_endpoint is None:
            agent_scheme = (
                "unix://"
                if os.getenv("ComputePlane__AgentChannel__SocketType", "unixdomainsocket")
                == "unixdomainsocket"
                else "http://"
            )
            agent_endpoint = agent_scheme + os.getenv(
                "ComputePlane__AgentChannel__Address", "/cache/armonik_agent.sock"
            )
        if channel_options is None:
            channel_options = (("grpc.default_authority", "localhost"),)
        self.logger = logger
        self.worker_endpoint = worker_endpoint
        self.agent_endpoint = agent_endpoint
        self.channel_options = channel_options
        self.processor = processor

    def __call__(self, *args, **kwargs):
        return self.processor(*args, **kwargs)

    def run(
        self,
        agent_channel: Optional[Channel] = None,
        logger: Optional[Logger] = None,
        worker_endpoint: Optional[str] = None,
    ):
        """
        Run the server
        Args:
            agent_channel: Agent channel
            logger: Logger
            worker_endpoint: Worker endpoint

        Returns:
            None
        """
        logger = self.logger if logger is None else logger
        worker_endpoint = self.worker_endpoint if worker_endpoint is None else worker_endpoint
        # Start worker
        logger.info("Worker Started")
        agent_channel = (
            create_channel(self.agent_endpoint, options=self.channel_options)
            if agent_channel is None
            else nullcontext(agent_channel)
        )

        with agent_channel as channel:
            worker = ArmoniKWorker(channel, self.processor, logger=logger)
            logger.info("Worker Connected")
            worker.start(worker_endpoint)


def armonik_worker(
    *,
    autorun: bool = False,
    logger: Optional[Logger] = None,
    worker_endpoint: Optional[str] = None,
    agent_endpoint: Optional[str] = None,
    channel_options: Optional[Iterable[Tuple[str, str]]] = None,
):
    """
    Transforms the function into an ArmoniK Worker
    Args:
        autorun: if True, runs the processor instead of returning the function
        logger: Logger to use, if None will use the default ClefLogger
        worker_endpoint: Worker endpoint, if None will use the default from ComputePlane__WorkerChannel__SocketType and ComputePlane__WorkerChannel__Address
        agent_endpoint: Agent endpoint, if None will use the default from ComputePlane__AgentChannel__SocketType and ComputePlane__AgentChannel__Address
        channel_options: Options for the gRPC channel

    Returns:
        Worker function

    Example:
        >>> @armonik_worker()
        >>> def processor(task_handler: TaskHandler) -> Output:
        >>>    ...
        >>>    return Output()
    """

    def decorator(
        processor: Callable[[TaskHandler], Output],
    ) -> ArmoniKWorkerWrapper:
        worker = ArmoniKWorkerWrapper(
            processor=processor,
            logger=logger,
            worker_endpoint=worker_endpoint,
            agent_endpoint=agent_endpoint,
            channel_options=channel_options,
        )
        if autorun:
            worker.run()
        return worker

    return decorator



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class TaskSummaryEnumField(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    TASK_SUMMARY_ENUM_FIELD_UNSPECIFIED: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_TASK_ID: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_SESSION_ID: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_OWNER_POD_ID: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_INITIAL_TASK_ID: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_STATUS: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_CREATED_AT: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_SUBMITTED_AT: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_STARTED_AT: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_ENDED_AT: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_CREATION_TO_END_DURATION: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_PROCESSING_TO_END_DURATION: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_RECEIVED_TO_END_DURATION: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_POD_TTL: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_POD_HOSTNAME: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_RECEIVED_AT: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_ACQUIRED_AT: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_PROCESSED_AT: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_ERROR: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_FETCHED_AT: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_PAYLOAD_ID: _ClassVar[TaskSummaryEnumField]
    TASK_SUMMARY_ENUM_FIELD_CREATED_BY: _ClassVar[TaskSummaryEnumField]

class TaskOptionEnumField(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    TASK_OPTION_ENUM_FIELD_UNSPECIFIED: _ClassVar[TaskOptionEnumField]
    TASK_OPTION_ENUM_FIELD_MAX_DURATION: _ClassVar[TaskOptionEnumField]
    TASK_OPTION_ENUM_FIELD_MAX_RETRIES: _ClassVar[TaskOptionEnumField]
    TASK_OPTION_ENUM_FIELD_PRIORITY: _ClassVar[TaskOptionEnumField]
    TASK_OPTION_ENUM_FIELD_PARTITION_ID: _ClassVar[TaskOptionEnumField]
    TASK_OPTION_ENUM_FIELD_APPLICATION_NAME: _ClassVar[TaskOptionEnumField]
    TASK_OPTION_ENUM_FIELD_APPLICATION_VERSION: _ClassVar[TaskOptionEnumField]
    TASK_OPTION_ENUM_FIELD_APPLICATION_NAMESPACE: _ClassVar[TaskOptionEnumField]
    TASK_OPTION_ENUM_FIELD_APPLICATION_SERVICE: _ClassVar[TaskOptionEnumField]
    TASK_OPTION_ENUM_FIELD_ENGINE_TYPE: _ClassVar[TaskOptionEnumField]
TASK_SUMMARY_ENUM_FIELD_UNSPECIFIED: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_TASK_ID: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_SESSION_ID: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_OWNER_POD_ID: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_INITIAL_TASK_ID: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_STATUS: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_CREATED_AT: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_SUBMITTED_AT: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_STARTED_AT: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_ENDED_AT: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_CREATION_TO_END_DURATION: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_PROCESSING_TO_END_DURATION: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_RECEIVED_TO_END_DURATION: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_POD_TTL: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_POD_HOSTNAME: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_RECEIVED_AT: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_ACQUIRED_AT: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_PROCESSED_AT: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_ERROR: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_FETCHED_AT: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_PAYLOAD_ID: TaskSummaryEnumField
TASK_SUMMARY_ENUM_FIELD_CREATED_BY: TaskSummaryEnumField
TASK_OPTION_ENUM_FIELD_UNSPECIFIED: TaskOptionEnumField
TASK_OPTION_ENUM_FIELD_MAX_DURATION: TaskOptionEnumField
TASK_OPTION_ENUM_FIELD_MAX_RETRIES: TaskOptionEnumField
TASK_OPTION_ENUM_FIELD_PRIORITY: TaskOptionEnumField
TASK_OPTION_ENUM_FIELD_PARTITION_ID: TaskOptionEnumField
TASK_OPTION_ENUM_FIELD_APPLICATION_NAME: TaskOptionEnumField
TASK_OPTION_ENUM_FIELD_APPLICATION_VERSION: TaskOptionEnumField
TASK_OPTION_ENUM_FIELD_APPLICATION_NAMESPACE: TaskOptionEnumField
TASK_OPTION_ENUM_FIELD_APPLICATION_SERVICE: TaskOptionEnumField
TASK_OPTION_ENUM_FIELD_ENGINE_TYPE: TaskOptionEnumField

class TaskSummaryField(_message.Message):
    __slots__ = ("field",)
    FIELD_FIELD_NUMBER: _ClassVar[int]
    field: TaskSummaryEnumField
    def __init__(self, field: _Optional[_Union[TaskSummaryEnumField, str]] = ...) -> None: ...

class TaskOptionField(_message.Message):
    __slots__ = ("field",)
    FIELD_FIELD_NUMBER: _ClassVar[int]
    field: TaskOptionEnumField
    def __init__(self, field: _Optional[_Union[TaskOptionEnumField, str]] = ...) -> None: ...

class TaskOptionGenericField(_message.Message):
    __slots__ = ("field",)
    FIELD_FIELD_NUMBER: _ClassVar[int]
    field: str
    def __init__(self, field: _Optional[str] = ...) -> None: ...

class TaskField(_message.Message):
    __slots__ = ("task_summary_field", "task_option_field", "task_option_generic_field")
    TASK_SUMMARY_FIELD_FIELD_NUMBER: _ClassVar[int]
    TASK_OPTION_FIELD_FIELD_NUMBER: _ClassVar[int]
    TASK_OPTION_GENERIC_FIELD_FIELD_NUMBER: _ClassVar[int]
    task_summary_field: TaskSummaryField
    task_option_field: TaskOptionField
    task_option_generic_field: TaskOptionGenericField
    def __init__(self, task_summary_field: _Optional[_Union[TaskSummaryField, _Mapping]] = ..., task_option_field: _Optional[_Union[TaskOptionField, _Mapping]] = ..., task_option_generic_field: _Optional[_Union[TaskOptionGenericField, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: agent_common.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from . import objects_pb2 as objects__pb2
from . import result_status_pb2 as result__status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x61gent_common.proto\x12\x19\x61rmonik.api.grpc.v1.agent\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\robjects.proto\x1a\x13result_status.proto\"\xc4\x02\n\x11\x43reateTaskRequest\x12P\n\x0cinit_request\x18\x01 \x01(\x0b\x32\x38.armonik.api.grpc.v1.agent.CreateTaskRequest.InitRequestH\x00\x12\x39\n\tinit_task\x18\x02 \x01(\x0b\x32$.armonik.api.grpc.v1.InitTaskRequestH\x00\x12\x36\n\x0ctask_payload\x18\x03 \x01(\x0b\x32\x1e.armonik.api.grpc.v1.DataChunkH\x00\x12\x1b\n\x13\x63ommunication_token\x18\x04 \x01(\t\x1a\x45\n\x0bInitRequest\x12\x36\n\x0ctask_options\x18\x01 \x01(\x0b\x32 .armonik.api.grpc.v1.TaskOptionsB\x06\n\x04type\"\xf7\x03\n\x0f\x43reateTaskReply\x12]\n\x14\x63reation_status_list\x18\x01 \x01(\x0b\x32=.armonik.api.grpc.v1.agent.CreateTaskReply.CreationStatusListH\x00\x12\x0f\n\x05\x65rror\x18\x02 \x01(\tH\x00\x12\x1b\n\x13\x63ommunication_token\x18\x04 \x01(\t\x1ah\n\x08TaskInfo\x12\x0f\n\x07task_id\x18\x01 \x01(\t\x12\x1c\n\x14\x65xpected_output_keys\x18\x02 \x03(\t\x12\x19\n\x11\x64\x61ta_dependencies\x18\x03 \x03(\t\x12\x12\n\npayload_id\x18\x04 \x01(\t\x1au\n\x0e\x43reationStatus\x12H\n\ttask_info\x18\x01 \x01(\x0b\x32\x33.armonik.api.grpc.v1.agent.CreateTaskReply.TaskInfoH\x00\x12\x0f\n\x05\x65rror\x18\x02 \x01(\tH\x00\x42\x08\n\x06Status\x1aj\n\x12\x43reationStatusList\x12T\n\x11\x63reation_statuses\x18\x01 \x03(\x0b\x32\x39.armonik.api.grpc.v1.agent.CreateTaskReply.CreationStatusB\n\n\x08Response\"=\n\x0b\x44\x61taRequest\x12\x1b\n\x13\x63ommunication_token\x18\x01 \x01(\t\x12\x11\n\tresult_id\x18\x02 \x01(\t\"!\n\x0c\x44\x61taResponse\x12\x11\n\tresult_id\x18\x02 \x01(\t\"\xc4\x01\n\x1c\x43reateResultsMetaDataRequest\x12U\n\x07results\x18\x01 \x03(\x0b\x32\x44.armonik.api.grpc.v1.agent.CreateResultsMetaDataRequest.ResultCreate\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\x1b\n\x13\x63ommunication_token\x18\x03 \x01(\t\x1a\x1c\n\x0cResultCreate\x12\x0c\n\x04name\x18\x01 \x01(\t\"\xb6\x01\n\x0eResultMetaData\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x11\n\tresult_id\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12?\n\x06status\x18\x04 \x01(\x0e\x32/.armonik.api.grpc.v1.result_status.ResultStatus\x12.\n\ncreated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"x\n\x1d\x43reateResultsMetaDataResponse\x12:\n\x07results\x18\x01 \x03(\x0b\x32).armonik.api.grpc.v1.agent.ResultMetaData\x12\x1b\n\x13\x63ommunication_token\x18\x02 \x01(\t\"\xe7\x02\n\x12SubmitTasksRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x36\n\x0ctask_options\x18\x02 \x01(\x0b\x32 .armonik.api.grpc.v1.TaskOptions\x12R\n\x0etask_creations\x18\x03 \x03(\x0b\x32:.armonik.api.grpc.v1.agent.SubmitTasksRequest.TaskCreation\x12\x1b\n\x13\x63ommunication_token\x18\x04 \x01(\t\x1a\x93\x01\n\x0cTaskCreation\x12\x1c\n\x14\x65xpected_output_keys\x18\x01 \x03(\t\x12\x19\n\x11\x64\x61ta_dependencies\x18\x02 \x03(\t\x12\x12\n\npayload_id\x18\x03 \x01(\t\x12\x36\n\x0ctask_options\x18\x04 \x01(\x0b\x32 .armonik.api.grpc.v1.TaskOptions\"\xe8\x01\n\x13SubmitTasksResponse\x12K\n\ntask_infos\x18\x01 \x03(\x0b\x32\x37.armonik.api.grpc.v1.agent.SubmitTasksResponse.TaskInfo\x12\x1b\n\x13\x63ommunication_token\x18\x02 \x01(\t\x1ag\n\x08TaskInfo\x12\x0f\n\x07task_id\x18\x01 \x01(\t\x12\x1b\n\x13\x65xpected_output_ids\x18\x02 \x03(\t\x12\x19\n\x11\x64\x61ta_dependencies\x18\x03 \x03(\t\x12\x12\n\npayload_id\x18\x04 \x01(\t\"\xc2\x01\n\x14\x43reateResultsRequest\x12M\n\x07results\x18\x01 \x03(\x0b\x32<.armonik.api.grpc.v1.agent.CreateResultsRequest.ResultCreate\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\x1b\n\x13\x63ommunication_token\x18\x03 \x01(\t\x1a*\n\x0cResultCreate\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"p\n\x15\x43reateResultsResponse\x12:\n\x07results\x18\x01 \x03(\x0b\x32).armonik.api.grpc.v1.agent.ResultMetaData\x12\x1b\n\x13\x63ommunication_token\x18\x02 \x01(\t\"\xc3\x01\n\x17NotifyResultDataRequest\x12P\n\x03ids\x18\x01 \x03(\x0b\x32\x43.armonik.api.grpc.v1.agent.NotifyResultDataRequest.ResultIdentifier\x12\x1b\n\x13\x63ommunication_token\x18\x04 \x01(\t\x1a\x39\n\x10ResultIdentifier\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x11\n\tresult_id\x18\x02 \x01(\t\".\n\x18NotifyResultDataResponse\x12\x12\n\nresult_ids\x18\x01 \x03(\tB?\n!fr.aneo.armonik.api.grpc.v1.agent\xaa\x02\x19\x41rmoniK.Api.gRPC.V1.Agentb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'agent_common_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n!fr.aneo.armonik.api.grpc.v1.agent\252\002\031ArmoniK.Api.gRPC.V1.Agent'
  _globals['_CREATETASKREQUEST']._serialized_start=119
  _globals['_CREATETASKREQUEST']._serialized_end=443
  _globals['_CREATETASKREQUEST_INITREQUEST']._serialized_start=366
  _globals['_CREATETASKREQUEST_INITREQUEST']._serialized_end=435
  _globals['_CREATETASKREPLY']._serialized_start=446
  _globals['_CREATETASKREPLY']._serialized_end=949
  _globals['_CREATETASKREPLY_TASKINFO']._serialized_start=606
  _globals['_CREATETASKREPLY_TASKINFO']._serialized_end=710
  _globals['_CREATETASKREPLY_CREATIONSTATUS']._serialized_start=712
  _globals['_CREATETASKREPLY_CREATIONSTATUS']._serialized_end=829
  _globals['_CREATETASKREPLY_CREATIONSTATUSLIST']._serialized_start=831
  _globals['_CREATETASKREPLY_CREATIONSTATUSLIST']._serialized_end=937
  _globals['_DATAREQUEST']._serialized_start=951
  _globals['_DATAREQUEST']._serialized_end=1012
  _globals['_DATARESPONSE']._serialized_start=1014
  _globals['_DATARESPONSE']._serialized_end=1047
  _globals['_CREATERESULTSMETADATAREQUEST']._serialized_start=1050
  _globals['_CREATERESULTSMETADATAREQUEST']._serialized_end=1246
  _globals['_CREATERESULTSMETADATAREQUEST_RESULTCREATE']._serialized_start=1218
  _globals['_CREATERESULTSMETADATAREQUEST_RESULTCREATE']._serialized_end=1246
  _globals['_RESULTMETADATA']._serialized_start=1249
  _globals['_RESULTMETADATA']._serialized_end=1431
  _globals['_CREATERESULTSMETADATARESPONSE']._serialized_start=1433
  _globals['_CREATERESULTSMETADATARESPONSE']._serialized_end=1553
  _globals['_SUBMITTASKSREQUEST']._serialized_start=1556
  _globals['_SUBMITTASKSREQUEST']._serialized_end=1915
  _globals['_SUBMITTASKSREQUEST_TASKCREATION']._serialized_start=1768
  _globals['_SUBMITTASKSREQUEST_TASKCREATION']._serialized_end=1915
  _globals['_SUBMITTASKSRESPONSE']._serialized_start=1918
  _globals['_SUBMITTASKSRESPONSE']._serialized_end=2150
  _globals['_SUBMITTASKSRESPONSE_TASKINFO']._serialized_start=2047
  _globals['_SUBMITTASKSRESPONSE_TASKINFO']._serialized_end=2150
  _globals['_CREATERESULTSREQUEST']._serialized_start=2153
  _globals['_CREATERESULTSREQUEST']._serialized_end=2347
  _globals['_CREATERESULTSREQUEST_RESULTCREATE']._serialized_start=2305
  _globals['_CREATERESULTSREQUEST_RESULTCREATE']._serialized_end=2347
  _globals['_CREATERESULTSRESPONSE']._serialized_start=2349
  _globals['_CREATERESULTSRESPONSE']._serialized_end=2461
  _globals['_NOTIFYRESULTDATAREQUEST']._serialized_start=2464
  _globals['_NOTIFYRESULTDATAREQUEST']._serialized_end=2659
  _globals['_NOTIFYRESULTDATAREQUEST_RESULTIDENTIFIER']._serialized_start=2602
  _globals['_NOTIFYRESULTDATAREQUEST_RESULTIDENTIFIER']._serialized_end=2659
  _globals['_NOTIFYRESULTDATARESPONSE']._serialized_start=2661
  _globals['_NOTIFYRESULTDATARESPONSE']._serialized_end=2707
# @@protoc_insertion_point(module_scope)



import datetime
import logging
import os

from armonik.common.channel import create_channel

from .conftest import data_folder, grpc_endpoint, ca_cert, client_cert, client_key
from armonik.worker import ArmoniKWorker, TaskHandler, ClefLogger
from armonik.common import Output, TaskOptions
from armonik.protogen.common.objects_pb2 import Empty, Configuration
from armonik.protogen.common.worker_common_pb2 import ProcessRequest


def do_nothing(_: TaskHandler) -> Output:
    return Output()


def throw_error(_: TaskHandler) -> Output:
    raise ValueError("TestError")


def return_error(_: TaskHandler) -> Output:
    return Output("TestError")


def return_and_send(th: TaskHandler) -> Output:
    th.send_results({th.expected_results[0]: b"result"})
    return Output()


class TestWorker:
    request = ProcessRequest(
        communication_token="token",
        session_id="session-id",
        task_id="task-id",
        expected_output_keys=["result-id"],
        payload_id="payload-id",
        data_dependencies=["dd-id"],
        data_folder=data_folder,
        configuration=Configuration(data_chunk_max_size=8000),
        task_options=TaskOptions(
            max_duration=datetime.timedelta(seconds=1), priority=1, max_retries=1
        ).to_message(),
    )

    def test_do_nothing(self):
        with create_channel(
            grpc_endpoint,
            certificate_authority=ca_cert,
            client_certificate=client_cert,
            client_key=client_key,
        ) as agent_channel:
            worker = ArmoniKWorker(
                agent_channel,
                do_nothing,
                logger=ClefLogger("TestLogger", level=logging.CRITICAL),
            )
            reply = worker.Process(self.request, None)
            assert Output(
                reply.output.error.details if reply.output.WhichOneof("type") == "error" else None
            ).success
            worker.HealthCheck(Empty(), None)

    def test_should_return_none(self):
        with create_channel(
            grpc_endpoint,
            certificate_authority=ca_cert,
            client_certificate=client_cert,
            client_key=client_key,
        ) as agent_channel:
            worker = ArmoniKWorker(
                agent_channel,
                throw_error,
                logger=ClefLogger("TestLogger", level=logging.CRITICAL),
            )
            reply = worker.Process(self.request, None)
            assert reply is None

    def test_should_error(self):
        with create_channel(
            grpc_endpoint,
            certificate_authority=ca_cert,
            client_certificate=client_cert,
            client_key=client_key,
        ) as agent_channel:
            worker = ArmoniKWorker(
                agent_channel,
                return_error,
                logger=ClefLogger("TestLogger", level=logging.CRITICAL),
            )
            reply = worker.Process(self.request, None)
            output = Output(
                reply.output.error.details if reply.output.WhichOneof("type") == "error" else None
            )
            assert not output.success
            assert output.error == "TestError"

    def test_should_write_result(self):
        with create_channel(
            grpc_endpoint,
            certificate_authority=ca_cert,
            client_certificate=client_cert,
            client_key=client_key,
        ) as agent_channel:
            worker = ArmoniKWorker(
                agent_channel,
                return_and_send,
                logger=ClefLogger("TestLogger", level=logging.DEBUG),
            )
            reply = worker.Process(self.request, None)
            assert reply is not None
            output = Output(
                reply.output.error.details if reply.output.WhichOneof("type") == "error" else None
            )
            assert output.success
            assert os.path.exists(os.path.join(data_folder, self.request.expected_output_keys[0]))
            with open(os.path.join(data_folder, self.request.expected_output_keys[0]), "rb") as f:
                value = f.read()
                assert len(value) > 0



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from ..common import sessions_common_pb2 as sessions__common__pb2


class SessionsStub(object):
    """*
    Service for handling sessions.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListSessions = channel.unary_unary(
                '/armonik.api.grpc.v1.sessions.Sessions/ListSessions',
                request_serializer=sessions__common__pb2.ListSessionsRequest.SerializeToString,
                response_deserializer=sessions__common__pb2.ListSessionsResponse.FromString,
                )
        self.GetSession = channel.unary_unary(
                '/armonik.api.grpc.v1.sessions.Sessions/GetSession',
                request_serializer=sessions__common__pb2.GetSessionRequest.SerializeToString,
                response_deserializer=sessions__common__pb2.GetSessionResponse.FromString,
                )
        self.CancelSession = channel.unary_unary(
                '/armonik.api.grpc.v1.sessions.Sessions/CancelSession',
                request_serializer=sessions__common__pb2.CancelSessionRequest.SerializeToString,
                response_deserializer=sessions__common__pb2.CancelSessionResponse.FromString,
                )
        self.CreateSession = channel.unary_unary(
                '/armonik.api.grpc.v1.sessions.Sessions/CreateSession',
                request_serializer=sessions__common__pb2.CreateSessionRequest.SerializeToString,
                response_deserializer=sessions__common__pb2.CreateSessionReply.FromString,
                )
        self.PauseSession = channel.unary_unary(
                '/armonik.api.grpc.v1.sessions.Sessions/PauseSession',
                request_serializer=sessions__common__pb2.PauseSessionRequest.SerializeToString,
                response_deserializer=sessions__common__pb2.PauseSessionResponse.FromString,
                )
        self.ResumeSession = channel.unary_unary(
                '/armonik.api.grpc.v1.sessions.Sessions/ResumeSession',
                request_serializer=sessions__common__pb2.ResumeSessionRequest.SerializeToString,
                response_deserializer=sessions__common__pb2.ResumeSessionResponse.FromString,
                )
        self.CloseSession = channel.unary_unary(
                '/armonik.api.grpc.v1.sessions.Sessions/CloseSession',
                request_serializer=sessions__common__pb2.CloseSessionRequest.SerializeToString,
                response_deserializer=sessions__common__pb2.CloseSessionResponse.FromString,
                )
        self.PurgeSession = channel.unary_unary(
                '/armonik.api.grpc.v1.sessions.Sessions/PurgeSession',
                request_serializer=sessions__common__pb2.PurgeSessionRequest.SerializeToString,
                response_deserializer=sessions__common__pb2.PurgeSessionResponse.FromString,
                )
        self.DeleteSession = channel.unary_unary(
                '/armonik.api.grpc.v1.sessions.Sessions/DeleteSession',
                request_serializer=sessions__common__pb2.DeleteSessionRequest.SerializeToString,
                response_deserializer=sessions__common__pb2.DeleteSessionResponse.FromString,
                )
        self.StopSubmission = channel.unary_unary(
                '/armonik.api.grpc.v1.sessions.Sessions/StopSubmission',
                request_serializer=sessions__common__pb2.StopSubmissionRequest.SerializeToString,
                response_deserializer=sessions__common__pb2.StopSubmissionResponse.FromString,
                )


class SessionsServicer(object):
    """*
    Service for handling sessions.
    """

    def ListSessions(self, request, context):
        """*
        Get a sessions list using pagination, filters and sorting.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSession(self, request, context):
        """*
        Get a session by its id.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CancelSession(self, request, context):
        """*
        Cancel a session by its id.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateSession(self, request, context):
        """*
        Create a session
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PauseSession(self, request, context):
        """*
        Pause a session by its id.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ResumeSession(self, request, context):
        """*
        Resume a paused session by its id.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CloseSession(self, request, context):
        """*
        Close a session by its id..
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PurgeSession(self, request, context):
        """*
        Purge a session by its id. Removes Results data.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteSession(self, request, context):
        """*
        Delete a session by its id. Removes metadata from Results, Sessions and Tasks associated to the session.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StopSubmission(self, request, context):
        """*
        Stops clients and/or workers from submitting new tasks in the given session.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SessionsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ListSessions': grpc.unary_unary_rpc_method_handler(
                    servicer.ListSessions,
                    request_deserializer=sessions__common__pb2.ListSessionsRequest.FromString,
                    response_serializer=sessions__common__pb2.ListSessionsResponse.SerializeToString,
            ),
            'GetSession': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSession,
                    request_deserializer=sessions__common__pb2.GetSessionRequest.FromString,
                    response_serializer=sessions__common__pb2.GetSessionResponse.SerializeToString,
            ),
            'CancelSession': grpc.unary_unary_rpc_method_handler(
                    servicer.CancelSession,
                    request_deserializer=sessions__common__pb2.CancelSessionRequest.FromString,
                    response_serializer=sessions__common__pb2.CancelSessionResponse.SerializeToString,
            ),
            'CreateSession': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateSession,
                    request_deserializer=sessions__common__pb2.CreateSessionRequest.FromString,
                    response_serializer=sessions__common__pb2.CreateSessionReply.SerializeToString,
            ),
            'PauseSession': grpc.unary_unary_rpc_method_handler(
                    servicer.PauseSession,
                    request_deserializer=sessions__common__pb2.PauseSessionRequest.FromString,
                    response_serializer=sessions__common__pb2.PauseSessionResponse.SerializeToString,
            ),
            'ResumeSession': grpc.unary_unary_rpc_method_handler(
                    servicer.ResumeSession,
                    request_deserializer=sessions__common__pb2.ResumeSessionRequest.FromString,
                    response_serializer=sessions__common__pb2.ResumeSessionResponse.SerializeToString,
            ),
            'CloseSession': grpc.unary_unary_rpc_method_handler(
                    servicer.CloseSession,
                    request_deserializer=sessions__common__pb2.CloseSessionRequest.FromString,
                    response_serializer=sessions__common__pb2.CloseSessionResponse.SerializeToString,
            ),
            'PurgeSession': grpc.unary_unary_rpc_method_handler(
                    servicer.PurgeSession,
                    request_deserializer=sessions__common__pb2.PurgeSessionRequest.FromString,
                    response_serializer=sessions__common__pb2.PurgeSessionResponse.SerializeToString,
            ),
            'DeleteSession': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteSession,
                    request_deserializer=sessions__common__pb2.DeleteSessionRequest.FromString,
                    response_serializer=sessions__common__pb2.DeleteSessionResponse.SerializeToString,
            ),
            'StopSubmission': grpc.unary_unary_rpc_method_handler(
                    servicer.StopSubmission,
                    request_deserializer=sessions__common__pb2.StopSubmissionRequest.FromString,
                    response_serializer=sessions__common__pb2.StopSubmissionResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'armonik.api.grpc.v1.sessions.Sessions', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Sessions(object):
    """*
    Service for handling sessions.
    """

    @staticmethod
    def ListSessions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.sessions.Sessions/ListSessions',
            sessions__common__pb2.ListSessionsRequest.SerializeToString,
            sessions__common__pb2.ListSessionsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSession(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.sessions.Sessions/GetSession',
            sessions__common__pb2.GetSessionRequest.SerializeToString,
            sessions__common__pb2.GetSessionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CancelSession(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.sessions.Sessions/CancelSession',
            sessions__common__pb2.CancelSessionRequest.SerializeToString,
            sessions__common__pb2.CancelSessionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateSession(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.sessions.Sessions/CreateSession',
            sessions__common__pb2.CreateSessionRequest.SerializeToString,
            sessions__common__pb2.CreateSessionReply.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def PauseSession(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.sessions.Sessions/PauseSession',
            sessions__common__pb2.PauseSessionRequest.SerializeToString,
            sessions__common__pb2.PauseSessionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ResumeSession(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.sessions.Sessions/ResumeSession',
            sessions__common__pb2.ResumeSessionRequest.SerializeToString,
            sessions__common__pb2.ResumeSessionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CloseSession(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.sessions.Sessions/CloseSession',
            sessions__common__pb2.CloseSessionRequest.SerializeToString,
            sessions__common__pb2.CloseSessionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def PurgeSession(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.sessions.Sessions/PurgeSession',
            sessions__common__pb2.PurgeSessionRequest.SerializeToString,
            sessions__common__pb2.PurgeSessionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteSession(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.sessions.Sessions/DeleteSession',
            sessions__common__pb2.DeleteSessionRequest.SerializeToString,
            sessions__common__pb2.DeleteSessionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StopSubmission(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/armonik.api.grpc.v1.sessions.Sessions/StopSubmission',
            sessions__common__pb2.StopSubmissionRequest.SerializeToString,
            sessions__common__pb2.StopSubmissionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from datetime import timedelta
import time
from threading import Thread

import grpc
import pytest
from armonik.client import ArmoniKTasks, ArmoniKResults, ArmoniKSessions, ArmoniKEvents
from armonik.common import TaskOptions, TaskDefinition

endpoint = ""


def wait_and_unpause(session_id: str):
    time.sleep(1)
    with grpc.insecure_channel(endpoint) as channel:
        ArmoniKSessions(channel).resume_session(session_id)
        print("Session resumed")


class TestWaitAvailability:
    def test_wait_availability(self):
        pytest.skip()
        n_tasks = 10000
        with grpc.insecure_channel(endpoint) as channel:
            task_client = ArmoniKTasks(channel)
            result_client = ArmoniKResults(channel)
            session_client = ArmoniKSessions(channel)
            events_client = ArmoniKEvents(channel)
            session_id = session_client.create_session(TaskOptions(timedelta(seconds=60), 1, 1, ""))
            print(f"Created session {session_id}")
            session_client.pause_session(session_id)
            payload_ids = list(
                r.result_id
                for r in result_client.create_results(
                    {str(r): str(r).encode() for r in range(n_tasks)}, session_id
                ).values()
            )
            print(f"Submitted payloads {len(payload_ids)}")
            result_ids = list(
                r.result_id
                for r in result_client.create_results_metadata(
                    [str(r) for r in range(n_tasks)], session_id
                ).values()
            )
            print(f"Submitted results {len(result_ids)}")
            tasks = task_client.submit_tasks(
                session_id,
                [
                    TaskDefinition(payload_id=p, expected_output_ids=[r])
                    for p, r in zip(payload_ids, result_ids)
                ],
            )
            print(f"Submitted tasks {len(tasks)}")
            t = Thread(target=wait_and_unpause, args=(session_id,))
            start = time.time()
            t.start()
            print("Waiting on results")
            events_client.wait_for_result_availability(result_ids, session_id, bucket_size=100)
            end = time.time()
            print(end - start)
            session_client.close_session(session_id)
            session_client.purge_session(session_id)
            session_client.delete_session(session_id)

    def test_wait_availability2(self):
        pytest.skip()
        n_tasks = 10000
        with grpc.insecure_channel(endpoint) as channel:
            task_client = ArmoniKTasks(channel)
            result_client = ArmoniKResults(channel)
            session_client = ArmoniKSessions(channel)
            events_client = ArmoniKEvents(channel)
            session_id = session_client.create_session(TaskOptions(timedelta(seconds=60), 1, 1, ""))
            print(f"Created session {session_id}")
            session_client.pause_session(session_id)
            payload_ids = list(
                r.result_id
                for r in result_client.create_results(
                    {str(r): str(r).encode() for r in range(n_tasks)}, session_id
                ).values()
            )
            print(f"Submitted payloads {len(payload_ids)}")
            result_ids = list(
                r.result_id
                for r in result_client.create_results_metadata(
                    [str(r) for r in range(n_tasks)], session_id
                ).values()
            )
            print(f"Submitted results {len(result_ids)}")
            tasks = task_client.submit_tasks(
                session_id,
                [
                    TaskDefinition(payload_id=p, expected_output_ids=[r])
                    for p, r in zip(payload_ids, result_ids)
                ],
            )
            print(f"Submitted tasks {len(tasks)}")
            t = Thread(target=wait_and_unpause, args=(session_id,))
            start = time.time()
            t.start()
            print("Waiting on results")
            events_client.wait_for_result_availability(
                result_ids, session_id, bucket_size=100, parallelism=10
            )
            end = time.time()
            print(end - start)
            session_client.close_session(session_id)
            session_client.purge_session(session_id)
            session_client.delete_session(session_id)



from google.protobuf import duration_pb2 as _duration_pb2
from . import task_status_pb2 as _task_status_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Empty(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class TaskOptions(_message.Message):
    __slots__ = ("options", "max_duration", "max_retries", "priority", "partition_id", "application_name", "application_version", "application_namespace", "application_service", "engine_type")
    class OptionsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    OPTIONS_FIELD_NUMBER: _ClassVar[int]
    MAX_DURATION_FIELD_NUMBER: _ClassVar[int]
    MAX_RETRIES_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_FIELD_NUMBER: _ClassVar[int]
    PARTITION_ID_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_NAME_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_VERSION_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_NAMESPACE_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_SERVICE_FIELD_NUMBER: _ClassVar[int]
    ENGINE_TYPE_FIELD_NUMBER: _ClassVar[int]
    options: _containers.ScalarMap[str, str]
    max_duration: _duration_pb2.Duration
    max_retries: int
    priority: int
    partition_id: str
    application_name: str
    application_version: str
    application_namespace: str
    application_service: str
    engine_type: str
    def __init__(self, options: _Optional[_Mapping[str, str]] = ..., max_duration: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ..., max_retries: _Optional[int] = ..., priority: _Optional[int] = ..., partition_id: _Optional[str] = ..., application_name: _Optional[str] = ..., application_version: _Optional[str] = ..., application_namespace: _Optional[str] = ..., application_service: _Optional[str] = ..., engine_type: _Optional[str] = ...) -> None: ...

class Session(_message.Message):
    __slots__ = ("id",)
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class Configuration(_message.Message):
    __slots__ = ("data_chunk_max_size",)
    DATA_CHUNK_MAX_SIZE_FIELD_NUMBER: _ClassVar[int]
    data_chunk_max_size: int
    def __init__(self, data_chunk_max_size: _Optional[int] = ...) -> None: ...

class Output(_message.Message):
    __slots__ = ("ok", "error")
    class Error(_message.Message):
        __slots__ = ("details",)
        DETAILS_FIELD_NUMBER: _ClassVar[int]
        details: str
        def __init__(self, details: _Optional[str] = ...) -> None: ...
    OK_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    ok: Empty
    error: Output.Error
    def __init__(self, ok: _Optional[_Union[Empty, _Mapping]] = ..., error: _Optional[_Union[Output.Error, _Mapping]] = ...) -> None: ...

class TaskRequest(_message.Message):
    __slots__ = ("expected_output_keys", "data_dependencies", "payload", "payload_name")
    EXPECTED_OUTPUT_KEYS_FIELD_NUMBER: _ClassVar[int]
    DATA_DEPENDENCIES_FIELD_NUMBER: _ClassVar[int]
    PAYLOAD_FIELD_NUMBER: _ClassVar[int]
    PAYLOAD_NAME_FIELD_NUMBER: _ClassVar[int]
    expected_output_keys: _containers.RepeatedScalarFieldContainer[str]
    data_dependencies: _containers.RepeatedScalarFieldContainer[str]
    payload: bytes
    payload_name: str
    def __init__(self, expected_output_keys: _Optional[_Iterable[str]] = ..., data_dependencies: _Optional[_Iterable[str]] = ..., payload: _Optional[bytes] = ..., payload_name: _Optional[str] = ...) -> None: ...

class InitKeyedDataStream(_message.Message):
    __slots__ = ("key", "last_result")
    KEY_FIELD_NUMBER: _ClassVar[int]
    LAST_RESULT_FIELD_NUMBER: _ClassVar[int]
    key: str
    last_result: bool
    def __init__(self, key: _Optional[str] = ..., last_result: bool = ...) -> None: ...

class DataChunk(_message.Message):
    __slots__ = ("data", "data_complete")
    DATA_FIELD_NUMBER: _ClassVar[int]
    DATA_COMPLETE_FIELD_NUMBER: _ClassVar[int]
    data: bytes
    data_complete: bool
    def __init__(self, data: _Optional[bytes] = ..., data_complete: bool = ...) -> None: ...

class TaskRequestHeader(_message.Message):
    __slots__ = ("expected_output_keys", "data_dependencies")
    EXPECTED_OUTPUT_KEYS_FIELD_NUMBER: _ClassVar[int]
    DATA_DEPENDENCIES_FIELD_NUMBER: _ClassVar[int]
    expected_output_keys: _containers.RepeatedScalarFieldContainer[str]
    data_dependencies: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, expected_output_keys: _Optional[_Iterable[str]] = ..., data_dependencies: _Optional[_Iterable[str]] = ...) -> None: ...

class InitTaskRequest(_message.Message):
    __slots__ = ("header", "last_task")
    HEADER_FIELD_NUMBER: _ClassVar[int]
    LAST_TASK_FIELD_NUMBER: _ClassVar[int]
    header: TaskRequestHeader
    last_task: bool
    def __init__(self, header: _Optional[_Union[TaskRequestHeader, _Mapping]] = ..., last_task: bool = ...) -> None: ...

class TaskId(_message.Message):
    __slots__ = ("session", "task")
    SESSION_FIELD_NUMBER: _ClassVar[int]
    TASK_FIELD_NUMBER: _ClassVar[int]
    session: str
    task: str
    def __init__(self, session: _Optional[str] = ..., task: _Optional[str] = ...) -> None: ...

class TaskIdList(_message.Message):
    __slots__ = ("task_ids",)
    TASK_IDS_FIELD_NUMBER: _ClassVar[int]
    task_ids: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, task_ids: _Optional[_Iterable[str]] = ...) -> None: ...

class StatusCount(_message.Message):
    __slots__ = ("status", "count")
    STATUS_FIELD_NUMBER: _ClassVar[int]
    COUNT_FIELD_NUMBER: _ClassVar[int]
    status: _task_status_pb2.TaskStatus
    count: int
    def __init__(self, status: _Optional[_Union[_task_status_pb2.TaskStatus, str]] = ..., count: _Optional[int] = ...) -> None: ...

class Count(_message.Message):
    __slots__ = ("values",)
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.RepeatedCompositeFieldContainer[StatusCount]
    def __init__(self, values: _Optional[_Iterable[_Union[StatusCount, _Mapping]]] = ...) -> None: ...

class ResultRequest(_message.Message):
    __slots__ = ("session", "result_id")
    SESSION_FIELD_NUMBER: _ClassVar[int]
    RESULT_ID_FIELD_NUMBER: _ClassVar[int]
    session: str
    result_id: str
    def __init__(self, session: _Optional[str] = ..., result_id: _Optional[str] = ...) -> None: ...

class TaskOutputRequest(_message.Message):
    __slots__ = ("session", "task_id")
    SESSION_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    session: str
    task_id: str
    def __init__(self, session: _Optional[str] = ..., task_id: _Optional[str] = ...) -> None: ...

class Error(_message.Message):
    __slots__ = ("task_status", "detail")
    TASK_STATUS_FIELD_NUMBER: _ClassVar[int]
    DETAIL_FIELD_NUMBER: _ClassVar[int]
    task_status: _task_status_pb2.TaskStatus
    detail: str
    def __init__(self, task_status: _Optional[_Union[_task_status_pb2.TaskStatus, str]] = ..., detail: _Optional[str] = ...) -> None: ...

class TaskError(_message.Message):
    __slots__ = ("task_id", "errors")
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    task_id: str
    errors: _containers.RepeatedCompositeFieldContainer[Error]
    def __init__(self, task_id: _Optional[str] = ..., errors: _Optional[_Iterable[_Union[Error, _Mapping]]] = ...) -> None: ...

class TaskList(_message.Message):
    __slots__ = ("task_ids",)
    TASK_IDS_FIELD_NUMBER: _ClassVar[int]
    task_ids: _containers.RepeatedCompositeFieldContainer[TaskId]
    def __init__(self, task_ids: _Optional[_Iterable[_Union[TaskId, _Mapping]]] = ...) -> None: ...

class TaskIdWithStatus(_message.Message):
    __slots__ = ("task_id", "status")
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    task_id: TaskId
    status: _task_status_pb2.TaskStatus
    def __init__(self, task_id: _Optional[_Union[TaskId, _Mapping]] = ..., status: _Optional[_Union[_task_status_pb2.TaskStatus, str]] = ...) -> None: ...



import os
from os import PathLike
from typing import Union, Tuple, Optional
from urllib.parse import urlparse

from grpc import Channel, secure_channel, insecure_channel, ssl_channel_credentials
from cryptography.hazmat.primitives.serialization import (
    pkcs12,
    Encoding,
    PrivateFormat,
    NoEncryption,
)


def _read_file(path: Union[str, PathLike]) -> bytes:
    with open(path, "rb") as f:
        return f.read()


_ca_cert_locations = [
    "/etc/ssl/certs/ca-certificates.crt",  # Debian/Ubuntu/Gentoo etc.
    "/etc/pki/tls/certs/ca-bundle.crt",  # Fedora/RHEL 6
    "/etc/ssl/ca-bundle.pem",  # OpenSUSE
    "/etc/pki/tls/cacert.pem",  # OpenELEC
    "/etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem",  # CentOS/RHEL 7
    "/etc/ssl/cert.pem",  # Alpine Linux
]

_cached_ca_cert_location: Optional[str] = ""


def _find_bundle_path() -> Optional[str]:
    global _cached_ca_cert_location
    if _cached_ca_cert_location is not None and len(_cached_ca_cert_location) > 0:
        return _cached_ca_cert_location
    # Look if one exists
    for location in _ca_cert_locations:
        if os.path.exists(location):
            _cached_ca_cert_location = location
            return location
    # Not found, will use the default provided
    _cached_ca_cert_location = None


def _load_certificates(
    certificate_authority: Union[str, PathLike, bytes, None] = None,
    client_certificate: Union[str, PathLike, bytes, None] = None,
    client_key: Union[str, PathLike, bytes, None] = None,
) -> Tuple[bytes, bytes, bytes]:
    if certificate_authority is None:
        certificate_authority = _find_bundle_path()  # Otherwise it uses the ca bundle of certifi. We want to find the installed ca bundle instead

    if certificate_authority is not None:
        if not isinstance(certificate_authority, bytes):
            certificate_authority = _read_file(certificate_authority)

    if client_certificate is not None:
        if not isinstance(client_certificate, bytes):
            client_certificate = _read_file(client_certificate)
        if client_key is None:
            # client and key are in the same file
            try:
                # Try to parse p12 if it's a p12
                pfx = pkcs12.load_pkcs12(client_certificate, b"")
                client_certificate = pfx.cert.certificate.public_bytes(Encoding.PEM)
                client_key = pfx.key.private_bytes(
                    Encoding.PEM, PrivateFormat.TraditionalOpenSSL, NoEncryption()
                )
            except ValueError:
                # Probably a PEM file
                client_key = client_certificate
        else:
            if not isinstance(client_key, bytes):
                client_key = _read_file(client_key)

    return certificate_authority, client_certificate, client_key


def create_channel(
    uri: str,
    *,
    options: Union[Tuple[Tuple[str, str]]] = None,
    certificate_authority: Union[str, PathLike, bytes, None] = None,
    client_certificate: Union[str, PathLike, bytes, None] = None,
    client_key: Union[str, PathLike, bytes, None] = None,
) -> Channel:
    """
    Create a gRPC channel for communication with the ArmoniK control plane
    Args:
        uri: URI of the channel. Will start a secure channel if the scheme contains "https". If it contains "unix", uses a unix socket.
        options: Options to pass to the channel
        certificate_authority: Certificate authority path to read, or content as bytes
        client_certificate: Client certificate path to read, or content as bytes
        client_key: Client key path to read, or content as bytes. If set to None but client_certificate is not None, assumes the key is included with the certificate (p12 or PEM certificate)
    Returns:
        Channel: gRPC channel for communication
    """
    parsed = urlparse(uri)
    scheme = parsed.scheme if parsed.scheme != "" else "http"
    endpoint = (
        parsed.netloc + parsed.path
    )  # To support with or without scheme, and for paths for unix

    if "unix" in scheme:
        # gRPC supports unix:path and the path is then relative, if the scheme is unix://, then the path is absolute
        if endpoint.startswith("/"):
            endpoint = "unix://" + endpoint
        else:
            endpoint = "unix:" + endpoint

    if "https" in scheme:
        certificate_authority, client_certificate, client_key = _load_certificates(
            certificate_authority, client_certificate, client_key
        )

        return secure_channel(
            endpoint,
            ssl_channel_credentials(
                root_certificates=certificate_authority,
                private_key=client_key,
                certificate_chain=client_certificate,
            ),
            options=options,
        )
    else:
        return insecure_channel(endpoint, options=options)



from __future__ import annotations
from enum import IntEnum

from ..protogen.common.task_status_pb2 import (
    TASK_STATUS_UNSPECIFIED,
    TASK_STATUS_CREATING,
    TASK_STATUS_SUBMITTED,
    TASK_STATUS_DISPATCHED,
    TASK_STATUS_COMPLETED,
    TASK_STATUS_ERROR,
    TASK_STATUS_TIMEOUT,
    TASK_STATUS_CANCELLING,
    TASK_STATUS_CANCELLED,
    TASK_STATUS_PROCESSING,
    TASK_STATUS_PROCESSED,
    TASK_STATUS_RETRIED,
    TASK_STATUS_PENDING,
    TASK_STATUS_PAUSED,
)
from ..protogen.common.events_common_pb2 import (
    EVENTS_ENUM_UNSPECIFIED,
    EVENTS_ENUM_NEW_TASK,
    EVENTS_ENUM_TASK_STATUS_UPDATE,
    EVENTS_ENUM_NEW_RESULT,
    EVENTS_ENUM_RESULT_STATUS_UPDATE,
    EVENTS_ENUM_RESULT_OWNER_UPDATE,
)
from ..protogen.common.session_status_pb2 import (
    SessionStatus as RawSessionStatus,
    _SESSIONSTATUS,
    SESSION_STATUS_UNSPECIFIED,
    SESSION_STATUS_RUNNING,
    SESSION_STATUS_CANCELLED,
    SESSION_STATUS_PAUSED,
    SESSION_STATUS_CLOSED,
    SESSION_STATUS_PURGED,
    SESSION_STATUS_DELETED,
)
from ..protogen.common.result_status_pb2 import (
    ResultStatus as RawResultStatus,
    _RESULTSTATUS,
    RESULT_STATUS_UNSPECIFIED,
    RESULT_STATUS_CREATED,
    RESULT_STATUS_COMPLETED,
    RESULT_STATUS_ABORTED,
    RESULT_STATUS_DELETED,
    RESULT_STATUS_NOTFOUND,
)
from ..protogen.common.health_checks_common_pb2 import (
    HEALTH_STATUS_ENUM_UNSPECIFIED,
    HEALTH_STATUS_ENUM_HEALTHY,
    HEALTH_STATUS_ENUM_DEGRADED,
    HEALTH_STATUS_ENUM_UNHEALTHY,
)
from ..protogen.common.worker_common_pb2 import HealthCheckReply
from ..protogen.common.sort_direction_pb2 import SORT_DIRECTION_ASC, SORT_DIRECTION_DESC

# This file is necessary because the grpc types aren't considered proper types


class TaskStatus(IntEnum):
    UNSPECIFIED = TASK_STATUS_UNSPECIFIED
    CREATING = TASK_STATUS_CREATING
    SUBMITTED = TASK_STATUS_SUBMITTED
    DISPATCHED = TASK_STATUS_DISPATCHED
    COMPLETED = TASK_STATUS_COMPLETED
    ERROR = TASK_STATUS_ERROR
    TIMEOUT = TASK_STATUS_TIMEOUT
    CANCELLING = TASK_STATUS_CANCELLING
    CANCELLED = TASK_STATUS_CANCELLED
    PROCESSING = TASK_STATUS_PROCESSING
    PROCESSED = TASK_STATUS_PROCESSED
    RETRIED = TASK_STATUS_RETRIED
    PENDING = TASK_STATUS_PENDING
    PAUSED = TASK_STATUS_PAUSED


class EventTypes(IntEnum):
    UNSPECIFIED = EVENTS_ENUM_UNSPECIFIED
    NEW_TASK = EVENTS_ENUM_NEW_TASK
    TASK_STATUS_UPDATE = EVENTS_ENUM_TASK_STATUS_UPDATE
    NEW_RESULT = EVENTS_ENUM_NEW_RESULT
    RESULT_STATUS_UPDATE = EVENTS_ENUM_RESULT_STATUS_UPDATE
    RESULT_OWNER_UPDATE = EVENTS_ENUM_RESULT_OWNER_UPDATE

    @classmethod
    def from_string(cls, name: str):
        return getattr(cls, name.upper())


class SessionStatus(IntEnum):
    @staticmethod
    def name_from_value(status: RawSessionStatus) -> str:
        return _SESSIONSTATUS.values_by_number[status].name

    UNSPECIFIED = SESSION_STATUS_UNSPECIFIED
    RUNNING = SESSION_STATUS_RUNNING
    CANCELLED = SESSION_STATUS_CANCELLED
    PAUSED = SESSION_STATUS_PAUSED
    CLOSED = SESSION_STATUS_CLOSED
    PURGED = SESSION_STATUS_PURGED
    DELETED = SESSION_STATUS_DELETED


class ResultStatus(IntEnum):
    @staticmethod
    def name_from_value(status: RawResultStatus) -> str:
        return _RESULTSTATUS.values_by_number[status].name

    UNSPECIFIED = RESULT_STATUS_UNSPECIFIED
    CREATED = RESULT_STATUS_CREATED
    COMPLETED = RESULT_STATUS_COMPLETED
    ABORTED = RESULT_STATUS_ABORTED
    DELETED = RESULT_STATUS_DELETED
    NOTFOUND = RESULT_STATUS_NOTFOUND


class ServiceHealthCheckStatus(IntEnum):
    UNSPECIFIED = HEALTH_STATUS_ENUM_UNSPECIFIED
    HEALTHY = HEALTH_STATUS_ENUM_HEALTHY
    DEGRADED = HEALTH_STATUS_ENUM_DEGRADED
    UNHEALTHY = HEALTH_STATUS_ENUM_UNHEALTHY


class HealthCheckStatus(IntEnum):
    UNKNOWN = HealthCheckReply.UNKNOWN
    SERVING = HealthCheckReply.SERVING
    NOT_SERVING = HealthCheckReply.NOT_SERVING


class Direction(IntEnum):
    ASC = SORT_DIRECTION_ASC
    DESC = SORT_DIRECTION_DESC
